@@ -7,15 +7,16 @@ from rich.logging import RichHandler
 from .easyfinemap import EasyFinemap
 from .ldref import LDRef
 from .loci import Loci
-from .sumstat import SumStat
+
+# from .sumstat import SumStat
 
 __author__ = """Jianhua Wang"""
 __email__ = 'jianhua.mert@gmail.com'
-__version__ = '0.2.1'
+__version__ = '0.2.4'
 
 
 logging.basicConfig(
-    level=logging.NOTSET,
+    level=logging.WARNING,
     format="%(name)s - %(message)s",
     datefmt="[%X]",
     handlers=[RichHandler(rich_tracebacks=True, show_path=False)],
