@@ -8,9 +8,9 @@ Perform fine-mapping for a locus using the following methods:
         2.1.1. FINEMAP
         2.1.2. PAINTOR
         2.1.3. CAVIARBF
-        TODO: 2.1.4. SuSiE
+        2.1.4. SuSiE
     2.2. with annotation
-        TODO: 2.2.1. PolyFun+SuSiE
+        TODO: 2.2.1. PolyFun
 3. Support multiple causal variant
 4. Support conditional mode
 """
@@ -28,9 +28,10 @@ from rich.progress import BarColumn, MofNCompleteColumn, Progress, TextColumn, T
 
 from easyfinemap.constant import ColName
 from easyfinemap.ldref import LDRef
-from easyfinemap.sumstat import SumStat
+
+# from easyfinemap.sumstat import SumStat
 from easyfinemap.tools import Tools
-from easyfinemap.utils import io_in_tempdir
+from easyfinemap.utils import io_in_tempdir, make_SNPID_unique
 
 
 class EasyFinemap(object):
@@ -311,6 +312,54 @@ class EasyFinemap(object):
             caviar_res = pd.Series(caviar_res[1].values, index=caviar_input[ColName.SNPID].tolist())
             return caviar_res
 
+    @io_in_tempdir('./tmp/easyfinemap')
+    def run_susie(
+            self,
+            sumstats: pd.DataFrame,
+            ld_matrix: str,
+            sample_size: int,
+            max_causal: int = 1,
+            temp_dir: Optional[str] = None,
+            **kwargs,
+    ) -> pd.Series:
+        """
+        Run SuSiE.
+
+        Parameters
+        ----------
+        sumstats : pd.DataFrame
+            Summary statistics.
+        ld_matrix : str
+            Path to LD matrix.
+        sample_size : int
+            Sample size.
+        max_causal : int, optional
+            Maximum number of causal variants, by default 1
+
+        Returns
+        -------
+        pd.Series
+            The result of SuSiE.
+        """
+        susie_input = sumstats.copy()
+        susie_input[ColName.Z] = susie_input[ColName.BETA] / susie_input[ColName.SE]
+        susie_input[[ColName.SNPID, ColName.Z]].to_csv(f"{temp_dir}/susie.input", sep=" ", index=False, header=True)
+
+        import rpy2.robjects as ro
+        ro.r(f'''ld = read.csv('{ld_matrix}', sep=' ', header=FALSE)
+                list_len = length(ld)
+                ld = ld[-length(ld)]
+                ld = as.matrix(ld)
+                df = read.csv('{temp_dir}/susie.input', sep=' ', header=TRUE)
+                z = df$Z
+                prior = df$SNPVAR
+                library('susieR')
+                res = susie_rss(z, ld, n={sample_size}, L = {max_causal})
+                pip = res$pip''')
+        susie_input['pip'] = ro.r('pip')
+        susie_res = pd.Series(susie_input['pip'].values, index=susie_input[ColName.SNPID].tolist())
+        return susie_res
+
     def cond_sumstat(
         self,
         sumstats: pd.DataFrame,
@@ -498,7 +547,7 @@ class EasyFinemap(object):
             fm_input = sumstats.copy()
             out_sumstats = sumstats.copy()
 
-        allowed_methods = ["abf", "finemap", "paintor", "caviarbf"]
+        allowed_methods = ["abf", "finemap", "paintor", "caviarbf", "susie"]
         if "all" in methods:
             methods = allowed_methods
         fm_input_ol = fm_input.copy()
@@ -506,9 +555,10 @@ class EasyFinemap(object):
             if method == "abf":
                 abf_pp = self.run_abf(sumstats=fm_input, **kwargs)
                 out_sumstats[ColName.PP_ABF] = out_sumstats[ColName.SNPID].map(abf_pp)
-            elif method in ["finemap", "paintor", "caviarbf"]:
+            elif method in ["finemap", "paintor", "caviarbf", "susie"]:
                 ld_matrix = f"{temp_dir}/intersc.ld"
                 if not os.path.exists(ld_matrix):
+                    # TODO: reduce the number of SNPs when using paintor and caviarbf in multiple causal variant mode
                     fm_input_ol = self.prepare_ld_matrix(sumstats=fm_input, outprefix=f"{temp_dir}/intersc", **kwargs)
                 if method == "finemap":
                     finemap_pp = self.run_finemap(sumstats=fm_input_ol, ld_matrix=ld_matrix, **kwargs)
@@ -519,6 +569,9 @@ class EasyFinemap(object):
                 elif method == "caviarbf":
                     caviarbf_pp = self.run_caviarbf(sumstats=fm_input_ol, ld_matrix=ld_matrix, **kwargs)
                     out_sumstats[ColName.PP_CAVIARBF] = out_sumstats[ColName.SNPID].map(caviarbf_pp)
+                elif method == "susie":
+                    susie_pp = self.run_susie(sumstats=fm_input_ol, ld_matrix=ld_matrix, **kwargs)
+                    out_sumstats[ColName.PP_SUSIE] = out_sumstats[ColName.SNPID].map(susie_pp)
             else:
                 raise ValueError(f"Method {method} is not supported")
 
@@ -580,8 +633,7 @@ class EasyFinemap(object):
         threads : int, optional
             Number of threads, by default 1
         """
-        sumstats = SumStat(sumstats)
-        sumstats = sumstats.standarize()
+        sumstats = make_SNPID_unique(sumstats)
         if credible_threshold and credible_method is None and methods != ["all"] and len(methods) == 1:
             credible_method = methods[0]
         kwargs_list = []
@@ -621,8 +673,8 @@ class EasyFinemap(object):
                     progress.update(task, advance=1)
                     progress.refresh()
                     output.append(res.get())
-        output = pd.concat(output, ignore_index=True)
+        output_df = pd.concat(output, ignore_index=True)
         if outfile:
-            output.to_csv(outfile, sep="\t", index=False)
+            output_df.to_csv(outfile, sep="\t", index=False)
         else:
-            return output
+            return output_df
