@@ -6,7 +6,7 @@
 2. intersect the significant snps with the LD reference.
 3. make a plink file from the intersected snps.
 TODO: 4. calculate LD matrix from the plink file.
-TODO: 5. perform conditional analysis in conditional mode.
+5. perform conditional analysis in conditional mode.
 """
 
 import logging
@@ -177,8 +177,18 @@ class LDRef:
             if "{chrom}" in ldref_path:
                 inprefix = ldref_path.replace("{chrom}", str(chrom))
                 if not os.path.exists(f"{inprefix}.bed"):
-                    self.logger.warning(f"{inprefix}.bed not found.")
-                    continue
+                    if chrom == 23:
+                        inprefix = ldref_path.replace("{chrom}", "X")
+                        if os.path.exists(f"{inprefix}.bed"):
+                            self.logger.warning(f"chr{chrom} not found, use X instead.")
+                            params[0].append(inprefix)
+                            params[1].append(f"{outprefix}.chr{chrom}")
+                            params[2].append(mac)
+                        else:
+                            self.logger.warning(f"{inprefix}.bed not found.")
+                    else:
+                        self.logger.warning(f"{inprefix}.bed not found.")
+                        continue
                 else:
                     params[0].append(inprefix)
                     params[1].append(f"{outprefix}.chr{chrom}")
@@ -352,7 +362,12 @@ class LDRef:
         return overlap_sumstat
 
     @io_in_tempdir('./tmp/ldref')
-    def make_ld(self, ldref: str, outprefix: str, **kwargs,):
+    def make_ld(
+        self,
+        ldref: str,
+        outprefix: str,
+        **kwargs,
+    ):
         """
         Make the LD matrix.
 
